--------------------------------------------------------
-- CS 535 - Snort on a Chip
-- Author: Michael Attig (mea1@arl.wustl.edu)
-- Created: August 5th, 2003
-- File Name: alert_generator.vhd
-- File Description:  This component creates alert
--  messages based on a Snort rule that has been
--  seen.  It will also generate response messages for
--  read requests. 
-- 
--------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL ;


ENTITY alert_generator IS

  PORT (
    clk                         : IN STD_LOGIC ;
    reset_l                     : IN STD_LOGIC ;

    -- from statistics block
    counter_data                : IN STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    counter_data_strobe         : IN STD_LOGIC ;
    
    -- to wrappers
    tca_appl_in_int             : IN STD_LOGIC ;
    d_appl_in_int               : OUT STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    dataen_appl_in_int          : OUT STD_LOGIC ;
    sof_appl_in_int             : OUT STD_LOGIC ;
    soip_appl_in_int            : OUT STD_LOGIC ;
    eof_appl_in_int             : OUT STD_LOGIC ;
    sop_appl_in_int             : OUT STD_LOGIC
    );
END alert_generator ;


ARCHITECTURE alert OF alert_generator IS

---------------------------------------------------------------------
-- Component Declarations
---------------------------------------------------------------------

COMPONENT alert_generator_fsm
  PORT (
    clk                 : IN STD_LOGIC ;
    reset_l             : IN STD_LOGIC ;
    counter_data_strobe : IN STD_LOGIC ;

    -- to wrappers
    tca_appl_in_int     : IN STD_LOGIC ;
    dataen_appl_in_int  : OUT STD_LOGIC ;
    sof_appl_in_int     : OUT STD_LOGIC ;
    soip_appl_in_int    : OUT STD_LOGIC ;
    eof_appl_in_int     : OUT STD_LOGIC ;
    sop_appl_in_int     : OUT STD_LOGIC ;

    -- to fifo
    data_count          : IN STD_LOGIC_VECTOR(3 DOWNTO 0) ;
    full                : IN STD_LOGIC ;
    empty               : IN STD_LOGIC ;
    read_fifo           : OUT STD_LOGIC ;
    write_fifo          : OUT STD_LOGIC ;

    output_data_sel     : OUT STD_LOGIC_VECTOR(3 DOWNTO 0)
    );
END COMPONENT ;
  
COMPONENT fifo_16_by_32
  PORT (
    clk                 : IN STD_LOGIC ;
    sinit               : IN STD_LOGIC ;
    din                 : IN STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    wr_en               : IN STD_LOGIC ;
    rd_en               : IN STD_LOGIC ;
    dout                : OUT STD_LOGIC_VECTOR(31 DOWNTO 0) ;
    full                : OUT STD_LOGIC ;
    empty               : OUT STD_LOGIC ;
    data_count          : OUT STD_LOGIC_VECTOR(3 DOWNTO 0)
    );
END COMPONENT ;
    
-- Synplicity black box declaration
attribute syn_black_box : boolean;
attribute syn_black_box of fifo_16_by_32: component is true;

---------------------------------------------------------------------
-- Signal Declarations
---------------------------------------------------------------------  

SIGNAL output_data_sel                  : STD_LOGIC_VECTOR(3 DOWNTO 0) ;
SIGNAL data_mux_out, data_to_fifo       : STD_LOGIC_VECTOR(31 DOWNTO 0) ;

SIGNAL data_count                       : STD_LOGIC_VECTOR(3 DOWNTO 0) ;
SIGNAL full, empty                      : STD_LOGIC ;
SIGNAL read_fifo, write_fifo            : STD_LOGIC ;

SIGNAL not_rst                          : STD_LOGIC ;

SIGNAL data_to_fifo_sel                 : STD_LOGIC_VECTOR(1 DOWNTO 0) ;
SIGNAL data_from_fifo                   : STD_LOGIC_VECTOR(31 DOWNTO 0) ;

SIGNAL udp_length, ip_length, dest_port : STD_LOGIC_VECTOR(15 DOWNTO 0) ;
SIGNAL length_sel                       : STD_LOGIC ;
SIGNAL current_src_ip, alert_dest       : STD_LOGIC_VECTOR(31 DOWNTO 0) ;


BEGIN

-- Logic

-------------------------------------------------------------------------------
-- Multiplexors
-------------------------------------------------------------------------------

  data_mux_out <= counter_data  ;
  
  ip_length <= x"0020" WHEN length_sel = '1'
               ELSE x"0038" ;

  udp_length <= x"000C" WHEN length_sel = '1'
                ELSE x"0024" ;

  WITH output_data_sel SELECT
    --                        2 if last pkt
    d_appl_in_int <= x"00000322"                WHEN "0000",  -- atm hdr
                     x"4500" & ip_length        WHEN "0001",  -- ip version_tos_length
                     x"00000000"                WHEN "0010",  -- ip_id_flags
                     x"40110000"                WHEN "0011",  -- ip_ttl_protocol_checksum
                     current_src_ip             WHEN "0100",  -- ip_src_ip
                     alert_dest                 WHEN "0101",  -- ip_dest_ip
                     x"0000" & dest_port        WHEN "0110",  -- udp port
                     udp_length & x"0000"       WHEN "0111",  -- udp length
                     x"00000000"                WHEN "1000",  -- aal5 trailer
                     x"00000000"                WHEN "1001",  -- aal5 trailer2
                     data_from_fifo WHEN OTHERS ;
    

length_sel <= '1' ;


---------------------------------------------------------------------
-- Process Name: flops
-- Process Sensitive to: clk
-- Process description: This process describes a flop positioned
--  between the multiplexor and the fifo
---------------------------------------------------------------------  
flops: PROCESS(clk)
BEGIN
  IF (clk = '1' and clk'EVENT) THEN
    IF (reset_l = '0') THEN
      data_to_fifo   <= x"00000000" ;
    ELSE
      data_to_fifo <= data_mux_out ;
     END IF ;
  END IF ;
END PROCESS flops ;

current_src_ip <= x"7F000001" ;
alert_dest     <= x"80FC9933" ;         -- my computer 
dest_port      <= x"FEED" ;             -- ???

---------------------------------------------------------------------
-- Port Maps
---------------------------------------------------------------------

fsm: alert_generator_fsm
  PORT MAP (
    clk                         => clk,
    reset_l                     => reset_l,
    counter_data_strobe         => counter_data_strobe,

    tca_appl_in_int             => tca_appl_in_int,
    dataen_appl_in_int          => dataen_appl_in_int,
    sof_appl_in_int             => sof_appl_in_int,
    soip_appl_in_int            => soip_appl_in_int,
    eof_appl_in_int             => eof_appl_in_int,
    sop_appl_in_int             => sop_appl_in_int,

    data_count                  => data_count,
    full                        => full,
    empty                       => empty,
    read_fifo                   => read_fifo,
    write_fifo                  => write_fifo,
    
    output_data_sel             => output_data_sel
    );

not_rst <= NOT reset_l ;

fifo: fifo_16_by_32
  PORT MAP (
    clk                         => clk,
    sinit                       => not_rst,
    din                         => data_to_fifo,
    wr_en                       => write_fifo,
    rd_en                       => read_fifo,
    dout                        => data_from_fifo,
    full                        => full,
    empty                       => empty,
    data_count                  => data_count
    );
    
END alert ;
