--------------------------------------------------------
-- CS 535 - Snort on a Chip
-- Author: Michael Attig (mea1@arl.wustl.edu)
-- Created: August 5th, 2003
-- File Name: alert_generator_fsm.vhd
-- File Description:  This component creates alert
--  messages based on a Snort rule that has been
--  seen.  It will also generate response messages for
--  read requests. 
-- 
--------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL ;
 

ENTITY alert_generator_fsm IS
  PORT (
    clk                 : IN STD_LOGIC ;
    reset_l             : IN STD_LOGIC ;
    counter_data_strobe : IN STD_LOGIC ;

    -- to wrappers
    tca_appl_in_int     : IN STD_LOGIC ;
    dataen_appl_in_int  : OUT STD_LOGIC ;
    sof_appl_in_int     : OUT STD_LOGIC ;
    soip_appl_in_int    : OUT STD_LOGIC ;
    eof_appl_in_int     : OUT STD_LOGIC ;
    sop_appl_in_int     : OUT STD_LOGIC ;

    -- to fifo
    data_count          : IN STD_LOGIC_VECTOR(3 DOWNTO 0) ;
    full                : IN STD_LOGIC ;
    empty               : IN STD_LOGIC ;
    read_fifo           : OUT STD_LOGIC ;
    write_fifo          : OUT STD_LOGIC ;
    
    output_data_sel     : OUT STD_LOGIC_VECTOR(3 DOWNTO 0)
    );
END alert_generator_fsm ;


ARCHITECTURE fsm OF alert_generator_fsm IS

---------------------------------------------------------------------
-- Type Declaration
---------------------------------------------------------------------  

TYPE states IS (idle, pkt_cntr,
                send_pkt, atm_hdr, atm_hdr2, ip_ver,
                ip_id_flags, ip_ttl, ip_source_ip,
                ip_dest_ip, udp_port, udp_length,
                pkt_data, aal5_trailer, aal5_trailer2);

SIGNAL state, next_state        : states ;
  
  
BEGIN

-------------------------------------------------------------------------------
-- Process Name: latch_state
-- Process Sensitive To: clk
-- Process Description: This process is responsible for latching the next
--  state into the flops and resetting the state upon a reset_l.
-------------------------------------------------------------------------------
latch_state: PROCESS(clk)

BEGIN

  IF (clk = '1' AND clk'EVENT) THEN
    IF (reset_l = '0') THEN
      state <= idle ;
    ELSE
      state <= next_state ;
    END IF ;
  END IF ;

END PROCESS latch_state ;    

-------------------------------------------------------------------------------
-- Process Name: nextstate
-- Process Sensitive To: state, counter_data_strobe,
--                       tca_appl_in_int, data_count,
-- Process Description: This process is responsible for describing the logic
--  to generate the next state.
-------------------------------------------------------------------------------
nextstate: PROCESS(state, counter_data_strobe,
                   tca_appl_in_int, data_count)
BEGIN

  -- default assignment
  next_state <= state ;

  CASE state IS

    WHEN idle =>
      IF (counter_data_strobe = '1') THEN
        next_state <= pkt_cntr ;
      ELSE
        next_state <= idle ;
      END IF ;
        
    WHEN pkt_cntr => next_state <= send_pkt ;

    WHEN send_pkt =>
      IF (tca_appl_in_int = '0') THEN
        next_state <= send_pkt ;
      ELSE
        next_state <= atm_hdr ;
      END IF ;

    WHEN atm_hdr =>      next_state <= ip_ver ;              
    WHEN ip_ver =>       next_state <= ip_id_flags ;
    WHEN ip_id_flags =>  next_state <= ip_ttl ;
    WHEN ip_ttl =>       next_state <= ip_source_ip ;
    WHEN ip_source_ip => next_state <= ip_dest_ip ;
    WHEN ip_dest_ip =>   next_state <= udp_port ;
    WHEN udp_port =>     next_state <= udp_length ;
    WHEN udp_length =>   next_state <= pkt_data ;
    WHEN pkt_data =>
      IF (data_count = "0000") THEN
        next_state <= aal5_trailer ;
      ELSE
        next_state <= pkt_data ;
      END IF ;
    WHEN aal5_trailer =>
      next_state <= aal5_trailer2 ;
    WHEN aal5_trailer2 =>
      next_state <= idle ;      
    WHEN OTHERS =>
      next_state <= idle ;
  END CASE ;
  
END PROCESS nextstate ;

-------------------------------------------------------------------------------
-- Process Name: outputs
-- Process Sensitive To: state, data_count
-- Process Description: This process describes the logic to generate the
--  outputs.
-------------------------------------------------------------------------------
outputs: PROCESS(state, data_count)
BEGIN

-- default output values
dataen_appl_in_int      <= '1' ;
sof_appl_in_int         <= '0' ;
soip_appl_in_int        <= '0' ;
eof_appl_in_int         <= '0' ;
sop_appl_in_int         <= '0' ;
read_fifo               <= '0' ;
write_fifo              <= '0' ;
output_data_sel         <= "1111" ;


  CASE state IS

    WHEN idle =>
      dataen_appl_in_int <= '0' ;
    WHEN pkt_cntr =>
      dataen_appl_in_int <= '0' ;
      write_fifo         <= '1' ;
    WHEN send_pkt =>
      dataen_appl_in_int <= '0' ;
    WHEN atm_hdr =>
      dataen_appl_in_int <= '0' ;
      sof_appl_in_int    <= '1' ;
      output_data_sel    <= "0000" ;
    WHEN ip_ver =>
      soip_appl_in_int   <= '1' ;
      output_data_sel    <= "0001" ;
    WHEN ip_id_flags =>
      output_data_sel    <= "0010" ;
    WHEN ip_ttl =>
      output_data_sel    <= "0011" ;
    WHEN ip_source_ip =>
      output_data_sel    <= "0100" ;
    WHEN ip_dest_ip =>
      output_data_sel    <= "0101" ;      
    WHEN udp_port =>
      sop_appl_in_int    <= '1' ;
      output_data_sel    <= "0110" ;
    WHEN udp_length =>
      output_data_sel    <= "0111" ;
      read_fifo <= '1' ;    
    WHEN pkt_data =>
      read_fifo          <= '1' ;
      IF (data_count = "0000") THEN
        eof_appl_in_int  <= '1' ;
      END IF ;
    WHEN aal5_trailer =>
      output_data_sel    <= "1000" ;     
    WHEN aal5_trailer2 =>      
      output_data_sel    <= "1001" ; 
    WHEN OTHERS =>
      dataen_appl_in_int <= '0' ;
      
  END CASE ;
END PROCESS outputs ;
  
END fsm ;
